@@ -14,6 +14,7 @@ import requests
 import pandas as pd
 
 import backtrader as bt
+from alpaca_trade_api.entity import Aggs
 from alpaca_trade_api.polygon.entity import NY
 from backtrader.metabase import MetaParams
 from backtrader.utils.py3 import queue, with_metaclass
@@ -325,7 +326,7 @@ class AlpacaStore(with_metaclass(MetaSingleton, object)):
     def _t_streaming_listener(self, q, tmout=None):
         while True:
             trans = q.get()
-            self._transaction(trans)
+            self._transaction(trans.order)
 
     def _t_streaming_events(self, q, tmout=None):
         if tmout is not None:
@@ -411,11 +412,25 @@ class AlpacaStore(with_metaclass(MetaSingleton, object)):
                             _from=self.iso_date(start_dt),
                             to=self.iso_date(end_dt))
                 else:
-                    response = self.oapi.get_aggs(dataname,
-                                                  compression,
-                                                  granularity,
-                                                  self.iso_date(start_dt),
-                                                  self.iso_date(end_dt))
+                    # response = self.oapi.get_aggs(dataname,
+                    #                               compression,
+                    #                               granularity,
+                    #                               self.iso_date(start_dt),
+                    #                               self.iso_date(end_dt))
+                    # so get_aggs work nicely for days but not for minutes, and
+                    # it is not a documented API. barset on the other hand does
+                    # but we need to manipulate it to be able to work with it
+                    # smoothly and return data the same way polygon does
+                    response = self.oapi.get_barset(
+                        dataname,
+                        granularity,
+                        start=start_dt,
+                        end=end_dt)[dataname]._raw
+                    for bar in response:
+                        # Aggs are in milliseconds, we multiply by 1000 to
+                        # change seconds to ms
+                        bar['t'] *= 1000
+                    response = Aggs({"results": response})
             except AlpacaError as e:
                 print(str(e))
                 q.put(e.error_response)
@@ -450,9 +465,11 @@ class AlpacaStore(with_metaclass(MetaSingleton, object)):
                                       'low': 'min',
                                       'close': 'last',
                                       'volume': 'sum'})
+        # don't use dt.replace. use localize
+        # (https://stackoverflow.com/a/1592837/2739124)
         cdl = cdl.loc[
-              dtbegin.replace(tzinfo=pytz.timezone(NY)):
-              dtend.replace(tzinfo=pytz.timezone(NY))
+              pytz.timezone(NY).localize(dtbegin):
+              pytz.timezone(NY).localize(dtend)
               ].dropna(subset=['high'])
         records = cdl.reset_index().to_dict('records')
         for r in records:
@@ -543,7 +560,9 @@ class AlpacaStore(with_metaclass(MetaSingleton, object)):
 
     def order_create(self, order, stopside=None, takeside=None, **kwargs):
         okwargs = dict()
-        okwargs['symbol'] = order.data._dataname
+        # different data feeds may set _name or _dataname so we cover both
+        okwargs['symbol'] = order.data._name if order.data._name else \
+            order.data._dataname
         okwargs['qty'] = abs(int(order.created.size))
         okwargs['side'] = 'buy' if order.isbuy() else 'sell'
         okwargs['type'] = self._ORDEREXECS[order.exectype]
@@ -558,11 +577,14 @@ class AlpacaStore(with_metaclass(MetaSingleton, object)):
         # if order.exectype == bt.Order.StopTrail:
         #     okwargs['trailingStop'] = order.trailamount
 
-        # if stopside is not None:
-        #     okwargs['stopLoss'] = stopside.price
+        if stopside:
+            okwargs['stop_loss'] = {'stop_price': stopside.price}
 
-        # if takeside is not None:
-        #     okwargs['takeProfit'] = takeside.price
+        if takeside:
+            okwargs['take_profit'] = {'limit_price': takeside.price}
+
+        if stopside or takeside:
+            okwargs['order_class'] = "bracket"
 
         okwargs.update(**kwargs)  # anything from the user
 
@@ -570,19 +592,30 @@ class AlpacaStore(with_metaclass(MetaSingleton, object)):
         return order
 
     def _t_order_create(self):
+        def _check_if_transaction_occurred(order_id):
+            # a transaction may have happened and was stored. if so let's
+            # process it
+            tpending = self._transpend[order_id]
+            tpending.append(None)  # eom marker
+            while True:
+                trans = tpending.popleft()
+                if trans is None:
+                    break
+                self._process_transaction(order_id, trans)
         while True:
             try:
+                if self.q_ordercreate.empty():
+                    continue
                 msg = self.q_ordercreate.get()
                 if msg is None:
-                    break
-
+                    continue
                 oref, okwargs = msg
                 try:
                     o = self.oapi.submit_order(**okwargs)
                 except Exception as e:
                     self.put_notification(e)
                     self.broker._reject(oref)
-                    return
+                    continue
                 try:
                     oid = o.id
                 except Exception:
@@ -592,23 +625,24 @@ class AlpacaStore(with_metaclass(MetaSingleton, object)):
                         self.put_notification(
                             "General error from the Alpaca server")
                     self.broker._reject(oref)
-                    return
+                    continue
 
-                self._orders[oref] = oid
-                self.broker._submit(oref)
                 if okwargs['type'] == 'market':
                     self.broker._accept(oref)  # taken immediately
 
+                self._orders[oref] = oid
                 self._ordersrev[oid] = oref  # maps ids to backtrader order
+                _check_if_transaction_occurred(oid)
+                if o.legs:
+                    index = 1
+                    for leg in o.legs:
+                        self._orders[oref + index] = leg.id
+                        self._ordersrev[leg.id] = oref + index
+                        _check_if_transaction_occurred(leg.id)
+                self.broker._submit(oref)  # inside it submits the legs too
+                if okwargs['type'] == 'market':
+                    self.broker._accept(oref)  # taken immediately
 
-                # An transaction may have happened and was stored
-                tpending = self._transpend[oid]
-                tpending.append(None)  # eom marker
-                while True:
-                    trans = tpending.popleft()
-                    if trans is None:
-                        break
-                    self._process_transaction(oid, trans)
             except Exception as e:
                 print(str(e))
 
